
import argparse
import subprocess
from time import sleep

from notebuild.shell import run_shell, run_shell_list
from notejob.tasks import start
from notetool.tool.log import log

logger = log("notebuild")


class JobScheduler:
    def __init__(self):
        pass

    def start(self):
        logger.info("start")
        subprocess.check_output(
            'nohup notejob _start >>/notechats/logs/jobs/notejob-run-$(date +%Y-%m-%d).log 2>&1 &', shell=True)

    def _start(self):
        logger.info("start")
        start()

    def stop(self):
        logger.info("stop")

        port = subprocess.check_output("ps x | grep notejob", shell=True)

        for line in port.decode().strip('\n').split('\n'):
            a = [i for i in line.split(' ') if len(i) > 0]

            if 'notejob' in a[-2] and a[-1] in ('restart', 'start', '_start'):
                subprocess.check_output('kill -9 {}'.format(a[0]), shell=True)
                logger.info("kill zhe process \n{}\n".format(line))

    def restart(self):
        self.stop()
        sleep(3)
        self.start()


def command_line_parser():
    parser = argparse.ArgumentParser(description="Test")
    parser.add_argument('command')
    args = parser.parse_args()
    return args


def notejob():
    args = command_line_parser()
    job = JobScheduler()
    if args.command == 'run':
        job.start()
    elif args.command == '_start':
        job._start()
    elif args.command == 'start':
        job.start()
    elif args.command == 'stop':
        job.stop()
    elif args.command == 'restart':
        job.restart()
    else:
        logger.info("unknown")



import logging
from logging.handlers import TimedRotatingFileHandler

from apscheduler.schedulers.gevent import GeventScheduler
from flask import Flask
from flask_apscheduler import APScheduler
from flask_sqlalchemy import SQLAlchemy

from ..config import config
from .CuBackgroundScheduler import CuBackgroundScheduler
from .CuGeventScheduler import CuGeventScheduler

# scheduler=APScheduler()
# scheduler = APScheduler(scheduler=CuGeventScheduler())
scheduler = APScheduler(scheduler=CuBackgroundScheduler())

db = SQLAlchemy()


def create_app(config_name):
    config_name = 'production'
    app = Flask(__name__)
    app.config.from_object(config[config_name])
    config[config_name].init_app(app)

    logging.basicConfig(level=logging.ERROR)

    formatter = logging.Formatter(
        "[%(asctime)s][%(filename)s:%(lineno)d][%(levelname)s][%(thread)d] - %(message)s")

    info_handler = TimedRotatingFileHandler("%s/datas/logs/info.log" % config[config_name].BASEDIR, when="H",
                                            interval=1, backupCount=7, encoding="UTF-8", delay=False,
                                            utc=True)
    # info_handler.setLevel(logging.INFO)
    info_handler.filter = lambda record: record.levelno == logging.INFO
    app.logger.addHandler(info_handler)
    info_handler.setFormatter(formatter)

    error_handler = TimedRotatingFileHandler("%s/datas/logs/error.log" % config[config_name].BASEDIR, when="D",
                                             interval=1, backupCount=15, encoding="UTF-8", delay=False,
                                             utc=True)
    error_handler.setLevel(logging.ERROR)
    app.logger.addHandler(error_handler)
    error_handler.setFormatter(formatter)

    scheduler.app = app
    db.init_app(app)
    scheduler.init_app(app)
    scheduler.start()

    from .main import main as main_blueprint
    app.register_blueprint(main_blueprint)

    # 接口对接
    from .api import api as apis_bl
    app.register_blueprint(apis_bl, url_prefix='/api')

    return app



from datetime import datetime

from apscheduler.executors.pool import ProcessPoolExecutor, ThreadPoolExecutor
from apscheduler.jobstores.memory import MemoryJobStore
from apscheduler.schedulers.blocking import BlockingScheduler
from notestock.dataset.run import run_month
from notetool.tool.log import logger


def my_job(id='my_job'):
    logger.info(id, '-->', datetime.now())


job_stores = {
    'default': MemoryJobStore(),
    # 'default': SQLAlchemyJobStore(url='sqlite:///jobs-sqlite.db')
}

executors = {
    'default': ThreadPoolExecutor(20),
    'processpool': ProcessPoolExecutor(10)
}

job_defaults = {
    'coalesce': False,
    'max_instances': 3
}


def my_listener(event):
    if event.exception:
        print('任务出错了！！！！！！')
    else:
        print('任务照常运行...')


def start():
    scheduler = BlockingScheduler(
        jobstores=job_stores, executors=executors, job_defaults=job_defaults)
    # scheduler = BackgroundScheduler(
    #    jobstores=job_stores, executors=executors, job_defaults=job_defaults)

    # scheduler.add_job(watch_product,  'interval', seconds=120, args=['44434'])
    # scheduler.add_job(watch_product,  'interval', seconds=120, args=['44435'])
    scheduler.add_job(run_month, 'interval', seconds=120, args=[])

    try:
        scheduler.start()
        print(scheduler.state)
        # a = 1
    except (KeyboardInterrupt, SystemExit):
        pass



from functools import wraps

from flask import redirect, session

from ..datas.utils.json import api_return


def api_err_return(code=1, msg='', data=''):
    return code, msg, data


'''
接口 api返回
'''


def api_deal_return(func):
    @wraps(func)
    def gen_status(*args, **kwargs):
        try:
            result = func(*args, **kwargs)
            if type(result) == str:
                return api_return(errcode=0, errmsg=result)
            if type(result) == list or type(result) == dict:
                return api_return(errcode=0, errmsg='success', data=result)
            if type(result) == tuple:
                if len(result) == 2:
                    errmsg = result[0]
                    if errmsg is None or errmsg == "":
                        errmsg = 'success'
                    return api_return(errcode=0, errmsg=errmsg, data=result[1])
                else:
                    return api_return(errcode=result[0], errmsg=result[1], data=result[2])
        except Exception as e:
            error = str(e)
            return api_return(errcode=1, errmsg=error)
    return gen_status


def login_required(func):
    @wraps(func)
    def wrapper(*args, **kwargs):
        if 'is_login' not in session:
            return redirect('/check_pass?msg=需要验证密码')

        return func(*args, **kwargs)

    return wrapper




from ...app import db


class CronInfos(db.Model):
    __tablename__ = 'cron_infos'
    id = db.Column(db.Integer, primary_key=True)
    task_name = db.Column(db.String(64), nullable=False)
    task_keyword = db.Column(db.String(65), nullable=False, default='')
    run_date = db.Column(db.String(25), default='', doc='执行时间')
    day_of_week = db.Column(db.String(10), default='', doc='星期几')
    day = db.Column(db.String(20), default='', doc='号(日)')
    hour = db.Column(db.String(10), default='', doc='小时')
    minute = db.Column(db.String(10), default='', doc='分钟')
    second = db.Column(db.String(10), default='', doc='秒')
    req_url = db.Column(db.String(128), default='')
    status = db.Column(db.SMALLINT, default=True, doc='运行状态，0停止1运行中-1结束任务')




import os

from flask_migrate import Migrate
from flask_script import Manager, Shell

from app import create_app, db
from datas.model.cron_infos import CronInfos
from datas.model.job_log import JobLog
from datas.model.job_log_items import JobLogItems

app = create_app(os.getenv('FLASK_CONFIG') or 'default')

manager = Manager(app)

migrate = Migrate(app, db)


def make_shell_context():
    return dict(
        app=app,
        JobLog=JobLog,
        CronInfos=CronInfos,
        JobLogItems=JobLogItems
    )


manager.add_command("shell", Shell(make_context=make_shell_context))
#manager.add_command('db', MigrateCommand)


@manager.command
def test():
    # print("Hello world")
    # d = configs('job_log_counts')
    # print(d)
    # print(uuid.uuid1())
    '''
    sudo docker exec -it $DOCKER_ID /bin/bash -c 'cd /packages/detectron && python tools/train.py'
    '''
    pass


if __name__ == '__main__':
    # gunicorn -b 127.0.0.1:5000 -w 1 -k gevent manage:app
    manager.run()




from ...app import db


class JobLogItems(db.Model):
    __tablename__='job_log_items'
    id = db.Column(db.Integer,primary_key=True)
    log_id = db.Column(db.String(65),index=True,nullable=False)
    content = db.Column(db.TEXT,nullable=False,default='')



from configparser import ConfigParser


def configs(key=None):
    cp = ConfigParser()
    cp.read('conf.ini', encoding='utf-8')
    if key:
        return cp.get('default', key)
    is_single = cp.get('default', 'is_single')
    redis_host = cp.get('default', 'redis_host')
    redis_pwd = cp.get('default', 'redis_pwd')
    redis_db = cp.get('default', 'redis_db')
    cron_db_url = cp.get('default', 'cron_db_url')
    cron_job_log_db_url = cp.get('default', 'cron_job_log_db_url')
    redis_port = cp.get('default', 'redis_port')
    login_pwd = cp.get('default', 'login_pwd')
    error_notice_api_key = cp.get('default', 'error_notice_api_key')
    job_log_counts = cp.get('default', 'job_log_counts')
    api_access_token = cp.get('default', 'api_access_token')
    error_keyword = cp.get('default', "error_keyword")

    pz = {
        'is_single': is_single,
        'redis_host': redis_host,
        'redis_pwd': redis_pwd,
        'redis_db': redis_db,
        'cron_db_url': cron_db_url,
        'cron_job_log_db_url': cron_job_log_db_url,
        'redis_port': redis_port,
        'login_pwd': login_pwd,
        'error_notice_api_key': error_notice_api_key,
        'job_log_counts': job_log_counts,
        'api_access_token': api_access_token,
        'error_keyword': error_keyword
    }

    return pz
